# Utworzenie roboczego środowiska dla programów w pythonie
 W tym ćwiczeniu należy stworzyć prosty program do obsługi portów GPIO, a następnie skonfigurować serwer "message brokera MQTT" do sterowania GPIO. Ponadto należy utworzyć skrypt startowy dla własnych aplikacji.
 ## Instalacja wirtualnego/lokalnego środowiska python:
- Przejście do katalogu z utworzonym repozytorium git (repozytorium danej grupy laboratoryjnej utworzone np. na git.pg.edu.pl lub github) dla tej części laboratorium: 
```bash 
cd <sciezka_do_katalogu_repozytorium>
```
- Utworzenie lokalnego  środowiska: 
```bash 
virtualenv -p python3.6 virtenv
```
gdzie:   virtenv -to nazwa katalogu, w którym zostanie utworzone lokalne środowisko python
- katalog virtenv należy dodać do pliku .gitignore
```bash 
touch .gitignore
echo "virtenv" >> .gitignore
git add .gitignore
```
## Aktywacja lokalnego środowiska python: 
```bash
source virtenv/bin/activate
```
- Test poprawnej instalacji środowiska lokalnego: 
```bash
pip -V
```
![Instalacja lokalnego środowiska python](lab01/img/installVirtevn.gif "Instalacja lokalnego środowiska python")

# Sterowanie diodą LED
## Instalacja biblioteki GPIO w lokalnym środowisku python:
```bash 
pip install gpio
```
## Zapisanie używanych bibliotek w pliku:
```bash
pip freeze > requirements.txt
```
## Instalacja wymaganych bibliotek (w przypadku instalacji w nowym środowisku):
```bash
pip install -r requirements.txt
```
![Instalacja biblioteki gpio](lab01/img/pythonInstallGpio.gif "Instalacja biblioteki gpio")

## Utworzenie skryptu sterującego dioda LED:
- Podłączenie DIODY LED do devKit
![Opis pinów devKit](lab01/img/NUC_opis_pinow.png "Opis pinów devKit")
![breadboardLab1](lab01/img/ledButtonLab1.png "Schemat podłączenia diody i przycisku do devKit")
- Przykładowa aplikacja
```python
#!/usr/bin/env python3.6
import time
import gpio
print( "Test LED@PWM0\n")
gpio.setup(504, gpio.OUT)
while(True):
    gpio.set(504, 1)
    time.sleep(0.5)
    gpio.set(504, 0)
    time.sleep(0.5)
```

# Odczyt stanu przycisku:
 - gpio.setup(488, gpio.IN)
 - gpio.read(488) # I2C0_DATA@GPIO488
 - zmiana ustawień poziomu loggera z gpio: 
```python
gpio.log.setLevel(logging.INFO)
```
**Zadania (2pkt):**
- Wyświetlić informacje na terminalu, tyko wtedy gdy zmieni sie stan przycisku (narastające lub opadające zbocze)
- Zaświecić diodę kiedy wykryto narastające zbocze, wygasić diodę gdy wykryto opadające zbocze na przycisku.

# Uruchomienie kontenera z MQTT
- Usuniecie starego: 
```bash
docker rm mosquitto
```
- Pobranie aktualnego obrazu kontenera:
```bash 
docker pull eclipse-mosquitto:1.6.13
```
- Utworzenie nowego z domyślną konfiguracją (porty 1883, 9001): 
```bash
docker create -p 1883:1883 -p 9001:9001 --name mosquitto eclipse-mosquitto:1.6.13
```
- Uruchomienie kontenera:
```bash
docker start mosquitto
```
- Sprawdzenie działania:
  - Sprawdzenie otwartych portów w systemie
```bash
lsof -i | grep 1883
lsof -i | grep 9001
docker ps
```
  - Podłączenie do log'ów dockera:
  ```bash
  docker attach mosquitto
  ``` 
  - odłączenie do log'ów dockera :
 UWAGA: to nie działa w VSC: ^P^Q (hold the Ctrl, press P, press Q, release Ctrl), workaround: [lnik](https://github.com/microsoft/vscode/issues/35722#issuecomment-334802447)
![Tworzenie/uruchomienie kontenera Mosquitto](lab01/img/dockerMosquitto.gif "Tworzenie/uruchomienie kontenera Mosquitto")
# Komunikacja przez MQTT
- Instalacja biblioteki do MQTT w lokalnym środowisku python:
  ```bash
  pip install paho-mqtt
  ```
- odbiorca wiadomości z MQTT:
```python
 import paho.mqtt.client as mqtt
# The callback for when the client receives a CONNACK response from the server.
def on_connect(client, userdata, flags, rc):
    print("Connected with result code "+str(rc))
    # Subscribing in on_connect() means that if we lose the connection and
    # reconnect then subscriptions will be renewed.
    client.subscribe("led/#")

# The callback for when a PUBLISH message is received from the server.
def on_message(client, userdata, msg):
    print(msg.topic+" "+str(msg.payload))

client = mqtt.Client()
client.on_connect = on_connect
client.on_message = on_message
client.connect("localhost", 1883, 60)

# Blocking call that processes network traffic, dispatches callbacks and
# handles reconnecting.
# Other loop*() functions are available that give a threaded interface and a
# manual interface.
client.loop_forever()
```
- nadawca wiadomości w MQTT:
```python
import paho.mqtt.client as mqtt
import time

# def on_log(client, userdata, level, buf):
#     print("log: ",buf)

flag_connected = 0
def on_connect(client, userdata, flags, rc):
   global flag_connected
   flag_connected = 1
   print("Connected with result code "+str(rc))

def on_disconnect(client, userdata, rc):
   global flag_connected
   flag_connected = 0
   print("Disconnected with result code "+str(rc))

client = mqtt.Client()
client.on_connect = on_connect
client.on_disconnect = on_disconnect
#client.on_log=on_log
client.connect("localhost", 1883, 60)
client.loop_start()

while(True):
    if flag_connected == 1:
        # Publish message
        client.publish("led/504", "1")
        time.sleep(1)
        client.publish("led/504", "0")
        time.sleep(1)
    else:
        # Wait to reconnect
        time.sleep(5)
        client.reconnect()

```
**Zadanie: (2pkt)**
- Należy napisać oprogramowanie wg poniższego schamatu, które za pomocą serwera MQTT bedzie odbierac wiadomości z aplikacji button.py oraz odpowiednio nadawać wiadmosci do aplikacji led.py.
![Schamat zadania z MQTT](lab01/img/ztb_lab01_sch.png "Schamat zadania z MQTT")

# Autostart aplikacji
## Cron -scheduler systemowy
- Edycja uruchamianych skryptów:
```bash
crontab -e
```
- Zawartość crontab
```bash
SHELL=/bin/bash
@reboot        /root/ztb/tmux_script.sh
```
## Skrypt tmux_script.sh
```bash
#!/bin/bash
MY_PATH=$(dirname $(realpath -s $0))
# Create a new tmux session named pmk
SESSION_NAME="pmk"
/usr/bin/tmux new-session -d -s $SESSION_NAME}
/usr/bin/tmux set-option -t ${SESSION_NAME} remain-on-exit on
/usr/bin/tmux send-keys -t ${SESSION_NAME} "source /root/.bashrc" C-m
/usr/bin/tmux send-keys -t ${SESSION_NAME} "source /root/.profile" C-m
/usr/bin/tmux send-keys -t ${SESSION_NAME} "date" C-m
/usr/bin/tmux send-keys -t ${SESSION_NAME} "source virtenv/bin/active" C-m
/usr/bin/tmux send-keys -t ${SESSION_NAME} "echo 'tutaj uruchom swoja app1'" C-m
/usr/bin/tmux split-window -t ${SESSION_NAME}
/usr/bin/tmux send-keys -t ${SESSION_NAME} "echo 'tutaj uruchom swoja app2 w drugim panelu'" C-m
```
- Podłączenie do sesji tmux uruchomionej podczas startu systemu
```bash
tmux a
```
- Skróty tmux: 
    - podział pionowy okna: CTR+B "
    - podział poziomy okna: CTR+B %
    - nowe okno: CTR+B c
    - następne okno: CTR+B n
    - wyjście bez zamykania okna: CTR+B d
    - zamknięcie panelu: CTR+d
    - zmiana panelu: CTR+b strzalki
# Zadania:
- Połączenie z MQTT z innego NUC (odczyt stanu przycisku) (3pkt)
  
  <img src="lab01/img/ztb_lab_button_other_NUC_schem.png"  width="220">
- Sterowanie LED na innym NUC (1 pkt)
  
  <img src="lab01/img/ztb_lab_led_other_NUC_schem.png"  width="220">
- Sterowanie krzyżowe Button@NUC1->LED@NUC2, Button@NUC2->LED@NUC1 (1 pkt)
- Uruchomienie skryptów podczas startu systemu (1 pkt)
<!---  Dodatkowe: Sterowanie wieloma LED@NUCX...NUCY -->
