## Podłączenie układu BME280 do pinów DevKitNUC magistrali SMB(i2c):

![Breadboard BME280](lab02/img/lab2.png "Breadboard NUC+BME280")

## Dodanie BME280 do magistrali SMB(i2c):
 ![Ex. add BME280 sensor](lab02/img/bme280_test.gif "Przykład odp. prawidłowo podłączonego sensora BME280")
- Test prawidłowego podłączenia przewodów sensora BME280 z NucDevKit (na magistrali smb/i2c powinno wykryć urządzenie pod adresem 0x76):
 ```bash
modprobe i2c_dev
i2cdetect -y -r 7
 ```
- Aktywacja sterownika IIO dla sensora BME280 (adres 0x76@i2c magistrala 7) w systemie
```bash
modprobe i2c_dev
echo "bme280 0x76" > /sys/bus/i2c/devices/i2c-7/new_device
```
- Test prawidłowej aktywacji sterownika IIO dla sensora BME280 :
```bash
 dmesg | tail
 ls /sys/bus/iio/devices/iio\:device0/
 ```
- Odczytanie wartości temp./ciśnienia/wilgotności:
 ```bash
 cat /sys/bus/iio/devices/iio\:device0/in_temp_input
```
## Odczyt wartości ze sterownika IIO w pythonie
```python
#!/usr/bin/env python3.6
path = "/sys/bus/iio/devices/iio:device0/in_temp_input"
with open(path,'r',encoding = 'utf-8') as f:
    print(f.read())  # read the entire file
```

## Zadanie (6pkt.): 
 Dodać obsługę czujnika BME280 w serwisie MQTT (docker mosquitto z lab.1):
 - nadawać wiadomości MQTT z ustalonym interwałem (np. 5s) w topic 'sensors/bme280/' - aplikacja sensorsBME280.py (1pkt.)

  <img src="lab02/img/ztb_lab2_schem_bme280.png"  width="240">
  
 - odczytywać własne wiadomości z temp. z MQTT (aplikacja recvBME280.py) (1pkt.)
 - wysłać wiadomości MQTT z aktualną temp., kiedy wykryto opadający zbocze na przycisku (wciśnięcie) (zmodyfikować sendBME280.py, tak aby odbierał wiadomości MQTT wysyłane z aplikacji button.py z lab1) (1pkt.)

  <img src="lab02/img/ztb_lab2_schem_bme280_button.png"  width="240">

 - odczytać wiadomości z temp. z innych DevKitNUC (zmodyfikować recvBME280.py, tak aby dobierał wiadomości z kliku kanałów MQTT) (1pkt.)

    <img src="lab02/img/ztb_lab2_schem_bme280_button_otherNUC.png"  width="240">

- utwórz aplikacje alarmTemp.py: odczytuj wiadomości z topic 'sensors/bme280', gdy temp>28st to zapal LED wysyłając wiadomości na topic 'led/504'),gdy temp.<28st. LED wygaszona, skorzystaj z apliakacji led.py z lab1 (2pkt.)

    <img src="lab02/img/ztb_lab2_schem_bme280_alarm.png"  width="240">

## Utworzyć nowe oraz uruchomić kontenery Telegraf, Influx, Grafana
### Telegraf
![Docker telegraf](lab02/img/dockertelegrafS2.gif "Przykład prawidłowego utworzenie i uruchomienie kontenera telegraf")
- Utworzenie kontenera

    UWAGA: przed uruchomieniem komendy '''docker create ...''' w katalogu projektu musi znajdować się plik konfiguracyjny [telegraf.conf](telegraf.conf). Proszę zaznajomić sie z jego zawartością.
```bash
docker pull telegraf
 docker rm telegraf
docker create  -v $(pwd)/:/etc/telegraf:ro --name telegraf telegraf
```
 - Start kontenera
```bash
docker start telegraf
```
 - Podłączenie do uruchomionego kontenera w celu obserwacji wyjście (output)
```bash
docker attach telegraf
```
 - Restart (tylko jeśli zachodzi potrzeba aktualizacji pliku [telegraf.conf](telegraf.conf))
```bash
docker restart telegraf
```
### Influx
![Docker Influx](lab02/img/dockerInfulxS.gif "Przykład prawidłowego utworzenie i uruchomienie kontenera influxdb")
- Utworzenie
 ```bash
docker pull influxdb
docker rm influxdb && docker volume rm influxdb
docker create -p 8086:8086 -v influxdb:/var/lib/influxdb --name influxdb influxdb:1.7
docker run --rm \
      -e INFLUXDB_DB=sensors -e INFLUXDB_ADMIN_ENABLED=true \
      -e INFLUXDB_ADMIN_USER=admin -e INFLUXDB_ADMIN_PASSWORD=ztb123 \
      -e INFLUXDB_USER=telegraf -e INFLUXDB_USER_PASSWORD=telegraf \
      -v influxdb:/var/lib/influxdb \
      influxdb:1.7 /init-influxdb.sh

```
- Start kontenera
```bash
docker start influxdb
```
 - Podłączenie do uruchomionego kontenera w celu obserwacji wyjście (output)
```bash
docker attach influxdb
```


### Grafana
 - Utworzenie 
```bash
docker pull grafana/grafana
docker rm grafana 
docker volume create grafana-storage
docker create -p 3000:3000 --name=grafana -v grafana-storage:/var/lib/grafana -e "GF_SECURITY_ADMIN_PASSWORD=ztb123" -e "GF_AUTH_ANONYMOUS_ENABLED=TRUE" -e "GF_AUTH_ANONYMOUS_ORG_NAME=GUT" -e "GR_AUTH_DISABLE_LOGIN_FORM=TRUE" grafana/grafana
```
 - Start kontenera grafana
```bash
docker start grafana
```
 - Podłączenie do uruchomionego kontenera w celu obserwacji wyjście (std.output)
```bash
docker attach grafana
```
- Test uruchomionych kontenerów
```bash
docker ps
```
![Docker TIG](lab02/img/tig.gif "Przykład prawidłowego utworzenie i uruchomienie kontenerów TIG")

 - Podłączenie do interfejsu www Grafana ("http://153.19.49.102:502X" gdzie X nalezy zamienić na nr grupy):
   ASW1: http://153.19.49.102:5021/
   ASW2: http://153.19.49.102:5022/

   ASW8: http://153.19.49.102:5028/

  ![WWW GRAFANA](lab02/img/grafanaustawieniaDB.png "Interfejs www Grafana")

  By stworzyć nowe wykresy w grafanie należy się zalogować (przycisk w lewym dolnym rogu).
  Wybieramy Nowy->Dashboard i wstawiamy nowy panel. Nowe query tworzymy wybierając nazwę tablicy w "select measurement", oraz nazwę pomiaru z "field". 

  ![query](lab02/img/query.png "query")

### Zadania (4pkt.):
- Konfiguracja Grafany w celu wyświetlenia danych pomiarowych (3pkt.):
  - Skonfiguruj połączenie z bazą influx
  - Utwórz nowy dashboard wyświetlający temp.

- Skonfiguruj Telegraf w taki sposób, aby pobierał dane pomiarowe MQTT z innych DevNuc - część konfiguracji jest zakomentowana w pliku conf (skrót odkomentowania {Ctrl+K}+{Ctrl+U} ) (1pkt.) 
